/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.abstractfactory;

import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.ParticipantUsage;
import dk.rode.thesis.meta.model.Sequence;

/**
  * A <i>prototypical sequence factory</i> creates {@link Sequence sequences} 
  * based on a prototype supplied at construction time. <p>
  * 
  * This factory is thread-safe. <p>
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    StandardFactory
  * 
  * @param  <E> The type of values delivered by the created sequences.
  */
@ParticipantUsage(value = "ConcreteProduct", type = Sequence.class)
@Participant("ConcreteFactory")
public class PrototypicalSequenceFactory<E> 
     extends PrototypicalFactory<Sequence<E>> implements SequenceFactory<E,Void> {

    /**
      * Constructor. <p>
      * 
      * {@code prototype} is copied and this factory keeps no reference
      * to it.
      * 
      * @param  prototype The prototypical sequence to use; cannot be null.
      * @throws NullPointerException If {@code prototype} is null.
      */
    public PrototypicalSequenceFactory(Sequence<E> prototype) {
        super(prototype);
    }
    
    /**
      * 
      */
    @Override
    public Sequence<E> create() {
        return super.create();
    }
    
    /**
      * @param unused Never used.
      */
    public Sequence<E> createSequence(Void unused) {
        return this.create();
    }
        
}
