/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.abstractfactory;

import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.model.Sequence;

/**
  * A <i>sequence factory</i> provides a general  
  * {@link #createSequence(Object) creation method} to 
  * create initialised sequences in a single step. <p>
  * 
  * The type of created sequences and the arguments used in
  * the creation process is specific to the implementation. 
  *  
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @param  <E> The type of values delivered by the created sequences.
  * @param  <P> The type of argument required in the creation process of
  *             sequences. Use {@code Void} if no arguments are required.
  */
@Participant("AbstractFactory")
public interface SequenceFactory<E,P> {
    
    /**
      * Creates a new initialised {@link Sequence sequence} instance. <p>
      * 
      * @param  argument A context supplied argument to use in creation process,
      *                  if any. 
      * @return A new sequence instance; never null.
      */
    public Sequence<E> createSequence(P argument);
    
}
