/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */

/**
  * Implementations and examples of the <b>Builder</b> 
  * design pattern [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.97]. <p>
  *  
  * <b>Intent</b>:<ul><p><i>
  * 
  *    Separate the construction of a complex object from its 
  *    representation so that the same construction process can 
  *    create different representations.
  *    
  * </i></ul>
  *  
  * Here, the <b>Builder</b> participant is represented by the
  * {@link dk.rode.thesis.builder.ExpressionBuilder} and 
  * {@link dk.rode.thesis.builder.ComparableExpressionBuilder} types,
  * and the <b>ConcreteBuilder</b> participant by any implementation
  * of either of them. The builders build 
  * {@link dk.rode.thesis.interpreter.Expression} objects, and the {@code Expression} 
  * interface thus corresponds to the <b>Product</b> participant. The overall 
  * constructed <i>root expression</i> can be fetched using the 
  * {@link dk.rode.thesis.builder.ExpressionBuilder#getRootExpression()} 
  * method. <p>
  * 
  * Six concrete builders are provided, namely the 
  * {@link dk.rode.thesis.builder.StandardExpressionBuilder}, 
  * {@link dk.rode.thesis.builder.StandardComparableExpressionBuilder},
  * {@link dk.rode.thesis.builder.CountingExpressionBuilder}, 
  * {@link dk.rode.thesis.builder.CountingComparableExpressionBuilder},
  * {@link dk.rode.thesis.builder.TypedExpressionBuilder}, and
  * {@link dk.rode.thesis.builder.TypedComparableExpressionBuilder},
  * classes. The middle two performs the same functionality as the first two,
  * respectively, but also keeps track of the number of constructed 
  * expressions, inspired by the Gamma et al. example in [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.104]. 
  * The last two creates {@link dk.rode.thesis.interpreter.TypedExpression}
  * instances, but uses composition to allow another builder to perform the
  * actual creation. <p>
  * 
  * Finally, the <b>Director</b> participant is represented by the
  * {@link dk.rode.thesis.builder.Main} class, which uses
  * the builders to construct {@code Expression} objects. <p>
  * 
  * <a name="uml"></a><b>UML Class Diagram</b>:<p>
  * <img src="doc-files/Builder.png" border="0"><p>
  * 
  * <b>Implementation notes</b>:</br>
  * Overall idea based on the example from [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.189]. <p>
  * 
  * The {@code TypedExpressionBuilder} illustrates good usage of covariant return types,
  * but also the problems that can be associated with it (see the class documentation).
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a> 
  */
@Pattern(name         = "Builder", 
         scope        = Scope.Object,
         purpose      = Purpose.Creational,
         participants = {"Builder", "ConcreteBuilder", "Director", "Product"})
package dk.rode.thesis.builder;

import dk.rode.thesis.meta.Pattern;
import dk.rode.thesis.meta.Purpose;
import dk.rode.thesis.meta.Scope;

