/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.composite;

import java.util.List;

import dk.rode.thesis.decorator.SequenceDecorator;
import dk.rode.thesis.meta.model.Sequence;
import dk.rode.thesis.strategy.ObjectPolicy;
import dk.rode.thesis.strategy.SequencePolicy;
import dk.rode.thesis.strategy.StringablePolicy;

/**
  * Additional policies for formatting {@link SequenceDecorator} 
  * objects into char sequences (<b>not</b> part of the core Composite 
  * implementation). <p>
  *
  * <b>Implementation notes</b>:<br>
  * <b>This is NOT part of the core Composite functionality, merely 
  * to provide better log descriptions!</b><p>
  * 
  * See implementation notes for the {@link ObjectPolicy} enumeration.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    SequencePolicy
  */
public enum CompositePolicy implements StringablePolicy<Sequence<?>> {

    /**
      * Policy used to format string representations of
      * composite sequences that indicates the structure. <p>
      * 
      * Note, however, that the check for composite sequences
      * is done using {@code instanceof}, which will not discover
      * adapted, decorated, or proxied sequences!
      */
    VISIBLE {
       public CharSequence toString(Sequence<?> sequence) {
           if (sequence instanceof CompositeSequence) {
               CompositeSequence<?> cs = (CompositeSequence<?>)sequence;
               StringBuilder sb = new StringBuilder();
               sb.append(ObjectPolicy.NAME_OPEN.toString(cs));
               List<?> sequences = cs.getSequences();
               // Associated sequences...
               for (int i = 0, n = sequences.size(); i < n; i++) {
                   Sequence<?> s = (Sequence<?>)sequences.get(i);
                   sb.append(i);
                   sb.append(": ");
                   sb.append(this.toString(s)); // Recurse...
                   if (i < n - 1) {
                       sb.append("; ");
                   }
               }
               return sb.append(']');
               
           // Default...
           } else {
               return sequence.toString(SequencePolicy.SHORT);
           }
       }
    }; 
      
    /**
      * @return {@link dk.rode.thesis.strategy.StringablePolicy.Type#APPENDABLE}. 
      */
    public Type type() {
        return Type.APPENDABLE;
    }
        
}
