/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */

/**
  * Implementations and examples of the <b>Composite</b> 
  * design pattern [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.163]. <p>
  *  
  * <b>Intent</b>:<ul><p><i>
  * 
  *    Compose objects into tree structures to represent partwhole 
  *    hierarchies. Composite lets clients treat individual objects 
  *    and compositions of objects uniformly.
  *     
  * </i></ul>
  * 
  * Here, the <b>Composite</b> participant is represented by the
  * {@link dk.rode.thesis.composite.CompositeSequence} class.
  * It allows for {@link dk.rode.thesis.meta.model.Sequence} 
  * objects to be composed into tree like structures and sequences
  * thus represent both the <b>Component</b> (by inheriting the 
  * {@code Sequence} interface) and <b>Leaf</b> (contained in composites) 
  * participants. <p>
  * 
  * The <b>Client</b> participant is the test application, i.e. the 
  * {@link dk.rode.thesis.composite.Main} class. <p>
  * 
  * <a name="uml"></a><b>UML Class Diagram</b>:<br>
  * <img src="doc-files/Composite.png" border="0"><p>
  * 
  * <b>Implementation notes</b>:<br/>
  * This Composite implementation opts for type-safety over transparency
  * as discussed by Gamma et al. [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.167-168]. Child management 
  * is defined in the <b>Composite</b> ({@code CompositeSequence}) participant 
  * because it can never make sense for <b>Leaf</b> components ({@code Sequence}). 
  * Unlike C++, the {@code instanceof} operator in Java provides a safe way to 
  * cast to a given type. <p> 
  * 
  * No explicit parent references are maintained for {@code Sequence} objects 
  * contained in a {@code CompositeSequence}. Sharing cannot generally be 
  * recommended because sequences carry state that is very likely to be manipulated
  * through the {@code Sequence} interface, which {@code CompositeSequence} inherits. 
  * Explicit child ordering is possible by using a {@link dk.rode.thesis.composite.CompositeStrategy}, 
  * such as {@link dk.rode.thesis.composite.CompositeStrategy#DEPTH_FIRST}. <p>
  * 
  * A limitation in the {@code SequenceComposite} interface is that
  * each contained sequence must be declared to use or extend the same
  * type parameter at construction time. However, it may be possible to use 
  * the {@linkplain dk.rode.thesis.adapter Adapter} pattern to 
  * circumvent this. <p>
  * 
  * This package defines a basic abstract implementation of the 
  * {@code CompositeSequence} interface, namely the 
  * {@link dk.rode.thesis.composite.AbstractCompositeSequence}
  * class. It stores child sequences in a random-access list for fast 
  * traversal. Child-related operations are not thread-safe. The 
  * {@link dk.rode.thesis.composite.CharSequenceCompositeSequence} class
  * is an actual implementation of the {@code CompositeSequence} 
  * interface.
  *  
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a> 
  */
@Pattern(name         = "Composite", 
         scope        = Scope.Object,
         purpose      = Purpose.Structural,
         participants = {"Component", "Composite", "Leaf", "Client"})
package dk.rode.thesis.composite;

import dk.rode.thesis.meta.Pattern;
import dk.rode.thesis.meta.Purpose;
import dk.rode.thesis.meta.Scope;

