/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */
package dk.rode.thesis.facade;

import dk.rode.thesis.meta.log.Log;
import dk.rode.thesis.meta.log.LogFactory;
import dk.rode.thesis.meta.test.Testable;
import dk.rode.thesis.meta.util.Arguments;

/**
  * <b>Facade</b> tests.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
public class Main implements Testable {
    
    /**
      * Executes the <b>Facade</b> tests.
      * 
      * @param args The arguments to supply to {@link #test(Log, Arguments)},
      *             if any; can be null.
      */
    public static void main(String[] args) {
        new Main().test(LogFactory.getLog(), new Arguments(args));
    }

    /**
      * No-arg constructor.
      */
    public Main() {
        super();
    }
    
    /**
      * 
      */
    public boolean test(Log out, Arguments arguments) {
        out.heading("Facade tests...");
        
        if (arguments.size() > 0) {
            out.println("Supplied arguments: ", arguments);
        }

        // Fetch facade...
        MathFacade math = MathFacade.getInstance();

        out.println("Using math facade");
        for (int n = 1; n < 200; n++) {
            out.println(n, ": Prime number          = ", math.getNthPrimeNumber(n));
            out.println(n, ": Random prime          = ", math.getRandomPrimeNumber(n));
            out.println(n, ": Fibonacci number      = ", math.getNthFibonnaciNumber(n));
            out.println(n, ": Random Fibonacci      = ", math.getRandomFibonnaciNumber(n));
            out.println(n, ": Random bounded number = ", math.getRandomNumber(n));
            out.println(n, ": Random boolean        = ", math.getRandomBoolean());
            out.println(n, ": Random number         = ", math.getRandomNumber());
            out.println(n, ": Ackermann number      = (m,n) = (", n % 3, ",", n, ") = ", math.getAckermannNumber(n % 3, n)); // stack depth!
            out.println();
        } 
        
        out.heading("Facade tests succeeded!");
        
        return true;
    }
    
}
