/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */

/**
  * Implementations and examples of the <b>Facade</b> 
  * design pattern [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.185]. <p>
  *  
  * <b>Intent</b>:<ul><p><i>
  * 
  *    Provide a unified interface to a set of interfaces in a subsystem. 
  *    Facade defines a higherlevel interface that makes the subsystem 
  *    easier to use.
  *     
  * </i></ul>
  *
  * Here, the <b>Facade</b> participant is represented by the 
  * {@link dk.rode.thesis.facade.MathFacade} class. It supplies an
  * <i>an-easy-to-use</i> high-level interface to complex mathematical
  * functions that internally are implemented using various 
  * {@link dk.rode.thesis.meta.model.Sequence} types. <p>
  * 
  * Specific <b>SubsystemClass</b> participants defined in this package are 
  * {@link dk.rode.thesis.facade.AckermannSequence},  
  * {@link dk.rode.thesis.facade.FibonacciSequence}, 
  * {@link dk.rode.thesis.facade.RandomSequence}, and 
  * {@link dk.rode.thesis.facade.UnboundedRandomSequence}, 
  * while various classes from other packages are also used to 
  * represent sub-system classes, for example 
  * {@link dk.rode.thesis.iterator.SequenceIterator} and
  * {@link dk.rode.thesis.state.ReversiblePrimeSequence}. <p>
  * 
  * <a name="uml"></a><b>UML Class Diagram</b>:<p>
  * <img src="doc-files/Facade.png" border="0"><p>
  * 
  * <b>Implementation notes</b>:<br>
  * The small examples used throughout this thesis does not really
  * justify use of Facade; they are simply not complex enough. Still, 
  * this Facade implementation address several issues discussed by
  * Gamma et al. [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.185-193]: <ul>
  * 
  * <li> The mathematical functions implemented with {@code Sequence} types
  *      are too verbose and tedious to be implemented by any client, and hence
  *      this facade help provide the desired functionality, shielding clients 
  *      from the gory details. <p>
  * 
  * <li> The facade shield the client from knowing about specific sub-system
  *      classes, i.e. sequences. Some sequences are declared package private,
  *      others not. This illustrate the use of private and public sub-system
  *      classes as discussed by Gamma et al. [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.188]<p>
  * 
  * <li> The facade implementation use package private access to ensure
  *      certain sub-system classes are off-limit from clients. An example
  *      is the {@code AckermannSequence} class. <p> 
  * 
  * <li> The facade is implemented as a {@linkplain dk.rode.thesis.singleton Singleton}. <p>
  * 
  * </ul>
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a> 
  */
@Pattern(name         = "Facade", 
         scope        = Scope.Object,
         purpose      = Purpose.Structural,
         participants = {"Facade", "SubsystemClass"}) 
package dk.rode.thesis.facade;

import dk.rode.thesis.meta.Pattern;
import dk.rode.thesis.meta.Purpose;
import dk.rode.thesis.meta.Scope;
