/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */
package dk.rode.thesis.meta;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.LOCAL_VARIABLE;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;
import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Documented;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
  * Identifies a <b>participant</b> as described in [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.7] that is 
  * already represented by an implemented type, typically imported into
  * the pattern package to represent participant functionality defined
  * externally. <p>
  * 
  * This annotation does not declare a participant type, but identifies an 
  * existing type as a participant used by the annotated element. The
  * identified participant may represent an interface even for <i>concrete 
  * participant</i> types. If so, any implementation is considered a valid 
  * representative of the participant.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    Participant
  */
@Documented
@Inherited
@Retention(RUNTIME)
@Target({TYPE, METHOD, PARAMETER, FIELD, LOCAL_VARIABLE})
public @interface ParticipantUsage {

    /**
      * The roles this participant fulfil in the pattern. <p>
      * 
      * An index in this array corresponds to the given participant 
      * {@link #type() type} identified at the same index. However, 
      * if all roles uses the same class, it is sufficient to declare
      * just that class. Vice versa, if several types are related to
      * the same role, a single role suffice.
      */
    String[] value();
    
    /**
      * The classes implementing a given participant role. <p>
      * 
      * An index in this array corresponds to the given participant 
      * {@link #value() role} identified at the same index. However, 
      * if all roles uses the same class, it is sufficient to declare
      * just that class. Vice versa, if several types are related to
      * the same role, a single role suffice.
      */
    Class<?>[] type();
    
}
