/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.visitor;

import java.util.Date;

import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.model.Sequence;

/**
  * A <i>date valued visitable sequence</i> allows any 
  * {@link Sequence sequence} delivering {@code Date} values
  * to become {@link ValueVisitableSequence value} visitable using the
  * {@link SequenceValueVisitor#visitDateValued(Sequence, Object)} 
  * visitation method <b>and</b> is {@link TypeVisitableSequence type}
  * visitable for any known sub-interface of {@code Sequence} 
  * that is implemented by the sequence. <p>
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    IntegerValuedVisitableSequence
  * @see    StringValuedVisitableSequence
  * @see    ReflectiveVisitableSequence
  */
@Participant("ConcreteElement")
public class DateValuedVisitableSequence extends AbstractVisitableSequence<Date> {

    /**
      * Constructor.
      * 
      * @param sequence The decorated sequence that will become visitable;
      *                 cannot be null.
      * @throws NullPointerException If {@code sequence} is null.                  
      */
    public DateValuedVisitableSequence(Sequence<Date> sequence) {
        super(sequence);
    }
    
    /**
      * {@inheritDoc} 
      *  
      * The decorated sequence is <b>not</b> copied. 
      */
    public DateValuedVisitableSequence copy() {
        return new DateValuedVisitableSequence(this.getSequence(false));
    }
    
    /**
      * @see SequenceValueVisitor#visitDateValued(Sequence, Object)
      */
    public <P> void accept(SequenceValueVisitor<P> visitor, P argument) {
        visitor.visitDateValued(this.getSequence(false), argument); // no copy
    }

}
